// -*- C++ -*
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/HeavyHadrons.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Projections/MissingMomentum.hh"
#include "Rivet/Projections/DressedLeptons.hh"
#include "Rivet/Projections/PromptFinalState.hh"

namespace Rivet {

  /// @brief WWW cross-section measurement
  class ATLAS_2016_I1492320_2l2j : public Analysis {

  public:

    // Default constructor
    DEFAULT_RIVET_ANALYSIS_CTOR(ATLAS_2016_I1492320_2l2j);

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {
      // Initialise and register projections here

      //Charged leptons within acceptance
      PromptFinalState chLep_fid = PromptFinalState(FinalState(Cuts::abspid == PID::ELECTRON || Cuts::abspid == PID::MUON));
      PromptFinalState photon_fs = PromptFinalState(FinalState(Cuts::abspid == PID::PHOTON));
      DressedLeptons dressed_leps(photon_fs, chLep_fid, 0.1, Cuts::pT > 10*GeV, true, false);
      declare(dressed_leps, "DressedLeptons");

      //Jets, anti-kt 0.4
      VetoedFinalState fsJets(FinalState(Cuts::abseta < 7.0)); //final state for jet finding: veto leptons and neutrinos
      fsJets.vetoNeutrinos();
      fsJets.addVetoOnThisFinalState(photon_fs);
      fsJets.addVetoOnThisFinalState(chLep_fid);
      declare(FastJets(fsJets, FastJets::ANTIKT, 0.4), "Jets");

      // b hadrons for b-tagging
      declare(HeavyHadrons(Cuts::abseta < 2.5 && Cuts::pT > 5.*GeV), "Bhadrons");
      
      //missing momentum
      declare(MissingMomentum(), "MET");
      
      // Histograms
      _h_2l2j = bookCounter("d01-x01-y02");
            
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();

      //evaluate needed projections (they have to be defined in init() first)
      vector<DressedLepton> leps = apply<DressedLeptons>(event, "DressedLeptons").dressedLeptons();
      if (leps.size() < 2) 	vetoEvent;
      //sort the dressed leptons by pt of their constituent lepton (bare lepton pt)
      std::sort(leps.begin() , leps.end() , 
                [](const DressedLepton &l1, const DressedLepton &l2) { 
                     return (l1.constituentLepton().pT() > l2.constituentLepton().pT());
                });
      const Jets& jets = apply<FastJets>(event, "Jets").jetsByPt(Cuts::pT > 15*GeV);

      if (leps[0].pT() < 30*GeV || leps[0].abseta() > 2.5)  vetoEvent;
      if (leps[1].pT() < 30*GeV || leps[1].abseta() > 2.5)  vetoEvent;

      float minDRll = 999., mll = -1.0;
      for (size_t i = 0; i < leps.size(); ++i) {
        for (size_t j = i + 1; j < leps.size(); ++j) {
          float dR = deltaR(leps[i], leps[j]);
          if (dR < minDRll)  minDRll = dR;
          float mll_tmp = (leps[i].momentum() + leps[j].momentum()).mass();
          if ((mll < 0) || (mll_tmp < mll))  mll = mll_tmp;
        }        
      }
      if (minDRll < 0.1)  vetoEvent;
      if (mll < 40*GeV)   vetoEvent;

      float ch_l0 = PID::charge(leps[0].pdgId());
      float ch_l1 = PID::charge(leps[1].pdgId());
      if (ch_l0*ch_l1 < 0)  vetoEvent;
      
      if ( fabs(mll - 91.188*GeV) < 10*GeV && abs(leps[0].pdgId()) == 11 && abs(leps[1].pdgId()) == 11 )  vetoEvent;
      
      //now jet cuts
      if (jets.size() < 2)   vetoEvent;
      if (jets[0].pT() < 30*GeV || jets[0].abseta() > 2.5)  vetoEvent;
      if (jets[1].pT() < 20*GeV || jets[1].abseta() > 2.5)  vetoEvent;

      float minDRLepJets = 999.;
      foreach (const auto &jet, jets) {
        foreach (const auto &lep, leps) {
          float dR = deltaR(lep,jet);
          if (dR < minDRLepJets)  minDRLepJets = dR;
        }        
      }
      if (minDRLepJets < 0.3)  vetoEvent;

      if (deltaEta(jets[0].eta(), jets[1].eta()) > 1.5)  vetoEvent;

      double mjj = (jets[0].momentum() + jets[1].momentum()).mass();
      if (mjj < 65 || mjj > 105)  vetoEvent;

      // For each of the good jets, check whether any are b-jets
      int nBJets = 0;
      const Particles& Bhadrons = apply<HeavyHadrons>(event, "Bhadrons").bHadrons();
      foreach (const Jet& j, jets) {
        if (j.abseta() > 2.5)  continue; //outside acceptance of b-tagging
        bool isbJet=false;
        foreach (const auto& b, Bhadrons) {
          isbJet |= deltaR(j, b) < 0.3;
        }
        if (isbJet)  ++nBJets;
      }
      if (nBJets)  vetoEvent;

      const MissingMomentum& met = apply<MissingMomentum>(event, "MET");
      if (met.vectorEt().mod() < 55*GeV && (abs(leps[0].pdgId()) != 13 || abs(leps[1].pdgId()) != 13))  vetoEvent;

      _h_2l2j->fill(weight);

    }


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_h_2l2j, crossSection()/sumOfWeights()/femtobarn);
    }

    //@}

  private:

    /// @name Histograms
    //@{
    CounterPtr _h_2l2j;
    //@}

  };

  // This global object acts as a hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2016_I1492320_2l2j);
}
