// -*- C++ -*
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Projections/MissingMomentum.hh"
#include "Rivet/Projections/DressedLeptons.hh"
#include "Rivet/Projections/PromptFinalState.hh"

namespace Rivet {

  /// @brief WWW cross-section measurement
  class ATLAS_2016_I1492320_3l : public Analysis {

  public:

    // Default constructor
    DEFAULT_RIVET_ANALYSIS_CTOR(ATLAS_2016_I1492320_3l);

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {
      //init cut values
      ZMass = 91.1876; //GeV

      // Initialise and register projections here

      //Charged leptons within acceptance
      PromptFinalState chLep_fid = PromptFinalState(FinalState(Cuts::abspid == PID::ELECTRON || Cuts::abspid == PID::MUON));
      PromptFinalState photon_fs = PromptFinalState(FinalState(Cuts::abspid == PID::PHOTON));
      DressedLeptons dressed_leps(photon_fs, chLep_fid, 0.1, Cuts::pT > 20*GeV && Cuts::abseta < 2.5, true, false);
      declare(dressed_leps, "DressedLeptons");

      //Jets, anti-kt 0.4
      VetoedFinalState fsJets(FinalState(Cuts::abseta < 7.0)); //final state for jet finding: veto leptons and neutrinos
      fsJets.vetoNeutrinos();
      fsJets.addVetoOnThisFinalState(photon_fs);
      fsJets.addVetoOnThisFinalState(chLep_fid);
      declare(FastJets(fsJets, FastJets::ANTIKT, 0.4), "Jets");

      //missing momentum
      declare(MissingMomentum(), "MET");

      // Histograms
      _h_fiducial_3l = bookCounter("d01-x01-y01");
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
	
      const double weight = event.weight();

      //evaluate needed projections (they have to be defined in init() first)
      vector<DressedLepton> _vbs_lep= apply<DressedLeptons>(event, "DressedLeptons").dressedLeptons();
      const Jets& _vbs_jets = apply<FastJets>(event, "Jets").jetsByPt(Cuts::pT > 25*GeV && Cuts::abseta < 4.5);
      if (_vbs_jets.size() > 1)  vetoEvent;

      //sort the dressed leptons by pt of their constituent lepton (bare lepton pt)
      std::sort(_vbs_lep.begin(), _vbs_lep.end(), [](const DressedLepton &l1, const DressedLepton &l2) { 
        return (l1.constituentLepton().pT() > l2.constituentLepton().pT());
      });

      if (_vbs_lep.size() != 3)  vetoEvent;

      //determine nsfos pairs for channel classification
      int nSFOS = 0 ;
      for (size_t i = 0; i < _vbs_lep.size(); ++i) {
        float ch_l0 = PID::charge(_vbs_lep[i].pdgId());
        for (size_t j = i + 1; j < _vbs_lep.size(); ++j) {
          float ch_l1 = PID::charge(_vbs_lep[j].pdgId());
          if (_vbs_lep[i].abspid() == _vbs_lep[j].abspid() && ch_l0 * ch_l1 < 0.)  ++nSFOS;
        }
      }

      float mSFOS_MinDiff = -1., meeSS_MinDiff = -1., mSF_min = -1., minDRll = 999.;
      for (size_t i = 0; i < _vbs_lep.size(); ++i) {
        float ch_l0 = PID::charge(_vbs_lep[i].pdgId());
        for (size_t j = i + 1; j < _vbs_lep.size(); ++j) {
          float ch_l1 = PID::charge(_vbs_lep[j].pdgId());

          float dR = deltaR(_vbs_lep[i], _vbs_lep[j]);
          if (dR < minDRll)  minDRll = dR;

          if (_vbs_lep[i].abspid() != _vbs_lep[j].abspid())  continue;

          //Determine min and max for all mee same-sign pairs
          if ( _vbs_lep[i].abspid() == 11 && ch_l0*ch_l1 > 0) {
            double meeSS_tmp = (_vbs_lep[i].momentum() + _vbs_lep[j].momentum()).mass()*GeV;
            if (meeSS_MinDiff < 0 || (fabs(meeSS_tmp-ZMass) < fabs(meeSS_MinDiff-ZMass)))  meeSS_MinDiff = meeSS_tmp;
          }

          //Determine min for all same-flavor pairs
          double mSF_tmp = (_vbs_lep[i].momentum() + _vbs_lep[j].momentum()).mass()*GeV;
          if (mSF_min < 0 || mSF_tmp < mSF_min)  mSF_min = mSF_tmp;

          if (ch_l0 * ch_l1 > 0.)  continue;

          //Determine min and max for all mSFOS pairs
          double mSFOS_tmp = (_vbs_lep[i].momentum() + _vbs_lep[j].momentum()).mass()*GeV;
          if (mSFOS_MinDiff < 0 || (fabs(mSFOS_tmp-ZMass) < abs(mSFOS_MinDiff-ZMass)))  mSFOS_MinDiff = mSFOS_tmp;
        }
      }
      if (minDRll < 0.1)  vetoEvent;
      if (nSFOS==0 && mSF_min < 20*GeV) vetoEvent;
      if (nSFOS==0 && fabs(meeSS_MinDiff - ZMass) < 15.)  vetoEvent;
      if (nSFOS==1 && ((ZMass - mSFOS_MinDiff) < 35. && (mSFOS_MinDiff - ZMass) < 20.))  vetoEvent;
      if (nSFOS==2 && fabs(mSFOS_MinDiff - ZMass) < 20.)  vetoEvent;

      const Vector3& met = -1.0 * apply<MissingMomentum>(event, "MET").vectorEt();
      if (nSFOS==1 && met.mod() < 45*GeV)	 vetoEvent;
      if (nSFOS==2 && met.mod() < 55*GeV)  vetoEvent;

      float dPhi = deltaPhi((_vbs_lep[0].momentum() +	_vbs_lep[1].momentum() + _vbs_lep[2].momentum()),	met);
      if (dPhi < 2.5)  vetoEvent;

      _h_fiducial_3l->fill(weight);

    }


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_h_fiducial_3l, crossSection()/sumOfWeights()/femtobarn);
    }

    //@}

  private:

    /// @name Histograms
    //@{
    float ZMass;
    CounterPtr _h_fiducial_3l;
    
    //@}
  };

  // This global object acts as a hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2016_I1492320_3l);
}
